from itertools import count
from __aplicarUps import aplicarUps
from __composicion import composicion
from __expandir import expandir
from __hashA import hashA
from __hashB import hashB


def crearfuncion_firmar (N, largo, zeta, M, comprimido, mbits, upbits):

	def firmar (mensaje, priv):
		mensajeBytes = mensaje if type (mensaje) == bytes else mensaje.encode ('utf-8')
		clave = expandir (priv['clave'], upbits, largo) if comprimido else priv['clave']
		sal = priv['sal']

		# aplicar hash con sal al mensaje
		h = hashA (sal + mensajeBytes, mbits)

		# usar un contador hasta obtener una composicion valida
		for n in count ():
			nHex = b'%x' % n
			m = hashB (nHex + h, mbits)
			if m >= M: continue

			c = composicion (N, largo, indice=m)
			if max (c) <= zeta: break

		# aplicar ups
		ups = [zeta - n for n in c]
		firma = aplicarUps (clave, ups, sal=sal, inicio=True, zeta=zeta)

		return firma, nHex, sal

	return firmar


# ----------------------------------------------------------------
if __name__ == '__main__':

	N, largo, zeta = 10, 4, 10
	M, mbits = 1716, 11
	firmar = crearfuncion_firmar (N, largo, zeta, M, comprimido=True, mbits=mbits, upbits=64)

	priv = {'clave': b'abc', 'sal': b'123'}
	mensaje = 'abc'
	firma, n, sal = firmar (mensaje, priv)

	print ('firma = ' + repr (firma))

