from itertools import count
from __aplicarUps import aplicarUps
from __comprimir import comprimir
from __hashA import hashA
from __hashB import hashB
from __mixradix import mixradix
from __separar import separar


def crearfuncion_verificar (M, basesAB, basesCD, zetasAB, comprimido, cbits, mbits):

	largo = len (basesAB + basesCD)

	def verificar (mensaje, firma=b'', n=b'0', sal=b'', pub=b''):
		try:
			mensajeBytes = mensaje if type (mensaje) == bytes else mensaje.encode ('utf-8')

			# aplicar hash con sal al mensaje
			h = hashA (sal + mensajeBytes, mbits)

			# aplicar hash con el contador al mensaje
			m = hashB (n + h, mbits)
			if m >= M: return False

			# calcular mensaje
			upsAB = mixradix (m, basesAB)
			downsAB = [zn - u for u, zn in zip (upsAB, zetasAB)]

			# calcular checksum
			check = sum (downsAB)
			upsCD = mixradix (check, basesCD)

			# calcular ups
			ups = upsAB + upsCD

			# aplicar ups
			f = separar (firma, largo)
			p = aplicarUps (f, ups=ups, sal=sal, inicio=False)

			# testear
			test = comprimir (p, cbits) if comprimido else p
			valido = (test == pub)
			return valido

		except:
			return False

	return verificar


# ----------------------------------------------------------------
if __name__ == '__main__':

	M, mbits = 10000000000000000, 54
	basesAB = 10, 10, 10, 10, 10, 10, 10, 10
	basesCD = 10, 10, 10, 10, 10, 10, 10, 10
	zetas = 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9


	verificar = crearfuncion_verificar (M, basesAB, basesCD, zetas, comprimido=True, cbits=64, mbits=mbits)

	pub = b'123'
	mensaje = 'abc'
	firma = b'firma invalida'
	n = b'0'
	sal = b''

	v = verificar (mensaje, firma=firma, n=n, sal=sal, pub=pub)

	print (v == False)

