from __b64 import b64
from __crearfuncion_claves import crearfuncion_claves
from __crearfuncion_firmar import crearfuncion_firmar
from __crearfuncion_verificar import crearfuncion_verificar
import __constantes as constantes


def uwots (bits=256, largo=28, comprimido=True, consal=True, cbits=512, upbits=None):

	# parametros

	if upbits == None: upbits = bits

	try:
		L = constantes.L[(bits, largo)]
		z = constantes.z[(bits, largo)]
	except:
		raise ValueError ('parametros incorrectos')

	L = list (L)
	for n in (0, 1, 2, 3):
		if z[n] == 0: L[n] = 0

	b = [n + 1 for n in z]
	basesAB = [b[0]] * L[0] + [b[1]] * L[1]
	basesCD = [b[2]] * L[2] + [b[3]] * L[3]

	zetasAB = [n - 1 for n in basesAB]
	zetasCD = [n - 1 for n in basesCD]
	zetas = zetasAB + zetasCD

	M = 1
	for base, largo in zip (b, L):
		M *= base ** largo

	mbits = (M - 1).bit_length ()

	# crear funciones

	crearclaves = crearfuncion_claves (zetas, comprimido, consal, cbits, upbits)
	firmar = crearfuncion_firmar (M, basesAB, basesCD, zetas, zetasAB, zetasCD, comprimido, mbits, upbits)
	verificar = crearfuncion_verificar (M, basesAB, basesCD, zetasAB, comprimido, cbits, mbits)

	return crearclaves, firmar, verificar


# ----------------------------------------------------------------
if __name__ == '__main__':

	crearclaves, firmar, verificar = uwots (bits=128, largo=14)

	# crear claves
	priv, pub = crearclaves ()

	# firmar
	mensaje = 'abc'
	firma, n, sal = firmar (mensaje, priv=priv)

	# verificar
	v = verificar (mensaje, firma, n, sal, pub=pub)

	print ('pub = ' + b64 (pub))
	print ('\nfirma = ' + b64 (firma))
	print ('n = ' + n.decode ('ascii'))
	print ('sal = ' + b64 (sal))
	print ('\nverificar = ' + str (v))

